/*
JSget - The 2D Full Featured Game Engine in JavaScript/HTML5

Author: Nathan Mentley (NathanMentley@GMail.com)
Version: 0.1
Updated: 10-08-2011
Web Site: http://NathanMentley.com/JSget/
File Info: JSgSSB.js contains an example Super Smash Bros style clone.
License: The Creative Commons Attribution-ShareAlike 3.0 Unported License

Full License Text:
	http://creativecommons.org/licenses/by-sa/3.0/legalcode

License Summary:
	You Are Free:
	to Share - to copy, distribute and transmit the work
	to Remix - to adapt the work
	To Profit - to make commercial use of the work

	Under the contidtions:
	Attribution — You must attribute the work in the manner specified by the author or
		licensor (but not in any way that suggests that they endorse you or your use
		of the work).
	Share Alike — If you alter, transform, or build upon this work, you may distribute the
		resulting work only under the same or similar license to this one.


	With the understanding that:
	Waiver — Any of the above conditions can be waived if you get permission from the
		copyright holder.
	Public Domain — Where the work or any of its elements is in the public domain under
		applicable law, that status is in no way affected by the license.
	Other Rights — In no way are any of the following rights affected by the license:
		Your fair dealing or fair use rights, or other applicable copyright exceptions
			and limitations;
		The author's moral rights;
		Rights other persons may have either in the work itself or in how the work is
			used, such as publicity or privacy rights.
*/
function Level(engine, name){
	this.engine = engine;

	this.name = name;
	this.levelImg = new Image();

	this.levelImg.src = this.engine.getFile("test.jpg", "image/jpg");
	this.render = function(){
		this.engine.drawImage(0,0,this.engine.WIDTH,this.engine.HEIGHT,this.levelImg);
	};
}

function Charecter(name){
	this.name = name;
	this.profileImg = new Image();
	this.profileImg.src = 'data/chars/' + this.name + "/profile.gif";
}

function Player(engine, x, y, w, h, ch){
	this.engine = engine;

	this.playerx = x;
	this.playery = y;
	this.playerh = h;
	this.playerw = w;

	this.playeryasc = 0;

	this.playerdmg = 0;
	this.playerlives = 3;

	this.cpu = true;
	this.ch = ch;

	this.image = new Image();
	this.image.src = "data/chars/firemario/0/0.gif";

	this.render = function(){
		this.engine.drawImage(this.playerx,this.playery,this.playerw,this.playerh,this.image);
	}
}

function GameData(engine){
	this.PLAYERSPEED = 10;
	this.GRAVITY = 1;
	this.PLAYERCOUNT = 4;
	this.CHARCOUNT = 2;
	this.LEVELCOUNT = 1;

	this.engine = engine;

	this.chars = new Array(this.CHARCOUNT);
	this.players = new Array(this.PLAYERCOUNT);
	this.levels = new Array(this.LEVELCOUNT);

	this.levelID = 0;

	for(var i = 0; i < this.CHARCOUNT;i++)
		this.chars[i] = new Charecter('mario');

	for(var i=0;i<this.PLAYERCOUNT;i++){
		this.players[i] = new Player(this.engine,100+i*50,100,30,45,this.chars[0]);
	}

	for(var i=0;i<this.LEVELCOUNT;i++){
		this.levels[i] = new Level(this.engine,'castle');
	}

	this.players[0].cpu = false;
}

function JSgGameState_Game(engine, gameData){
	this.prev = null;
	this.next = null;
	this.engine = engine;

	this.gameData = gameData;

	this.init = function(){
		this.doRender = true;
		this.doLogic = true;
	};
	this.deinit = function(){
		this.doRender = false;
		this.doLogic = false;
	};
	this.logic = function(key){
		for(var i = 0; i < this.gameData.PLAYERCOUNT; i++){
			this.gameData.players[i].playeryasc += this.gameData.GRAVITY;

			if(this.gameData.players[i].playery >= this.engine.HEIGHT - 50){
				this.gameData.players[i].playery = this.engine.HEIGHT - 50;
				if(this.gameData.players[i].playeryasc > 0){
					this.gameData.players[i].playeryasc = 0;
				}
			}
			this.gameData.players[i].playery += this.gameData.players[i].playeryasc;
		}

		if(this.engine.isKeyDown(27)) //esc
			this.engine.popState();
		if(this.engine.isKeyDown(38)){//up
			if(this.gameData.players[0].playeryasc < 1){
				if(this.gameData.players[0].playery >= this.engine.HEIGHT - 50)
					this.gameData.players[0].playeryasc -= 15;
			}
		}
		if(this.engine.isKeyDown(37)) //left
			if(this.gameData.players[0].playerx > 0)
				this.gameData.players[0].playerx -= this.gameData.PLAYERSPEED;
		if(this.engine.isKeyDown(39)) //right
			if(this.gameData.players[0].playerx < this.engine.WIDTH - 30)
				this.gameData.players[0].playerx += this.gameData.PLAYERSPEED;
		if(this.engine.isKeyDown(90)) //z
			this.gameData.players[0].playerx += this.gameData.PLAYERSPEED;
		if(this.engine.isKeyDown(88)) //x
			this.gameData.players[0].playerx += this.gameData.PLAYERSPEED;
		if(this.engine.isKeyDown(67)) //c
			this.gameData.players[0].playerx += this.gameData.PLAYERSPEED;
	
		if(this.engine.mouseClicked == true){
			this.engine.mouseClicked = false;
		}

		for(i = 0; i < this.gameData.PLAYERCOUNT; i++){
			if(this.gameData.players[i].cpu){
				if(this.gameData.players[i].playerx < this.gameData.players[0].playerx){
					this.gameData.players[i].playerx += this.gameData.PLAYERSPEED;
				}else{
					this.gameData.players[i].playerx -= this.gameData.PLAYERSPEED;
				}
			}
		}
	};
	this.render = function(){
		this.gameData.levels[this.gameData.levelID].render();
		for(var i = 0; i < this.gameData.PLAYERCOUNT; i++)
			this.gameData.players[i].render();
	};
	this.pause = function(){
		this.doRender = false;
		this.doLogic = false;
	};
	this.unpause = function(){
		this.doRender = true;
		this.doLogic = true;
	};
}

function JSgGameState_LevelSelect(engine, gameData){
	this.prev = null;
	this.next = null;
	this.engine = engine;

	this.gameData = gameData;

	this.bgimg = new Image();
	this.bgimg.src = 'data/test.jpg';
	this.backButton = new Image();
	this.backButton.src = 'data/button.jpg';
	this.linkButton = new Image();
	this.linkButton.src = 'data/link.jpg';

	this.init = function(){
		this.doRender = true;
		this.doLogic = true;
	};
	this.deinit = function(){
		this.doRender = false;
		this.doLogic = false;
	};
	this.logic = function(key){
		if(this.engine.mouseClicked == true){
			if(this.engine.mouseIn(400,430,100,20)){
				this.engine.popState();
			}
			if(this.engine.mouseIn(20,20,60,90)){
				this.engine.pushState(new JSgGameState_Game(this.engine, this.gameData));
			}
			this.engine.mouseClicked = false;
		}
	};
	this.render = function(){
		this.engine.drawImage(0,0,engine.WIDTH,engine.HEIGHT,this.bgimg);
		this.engine.drawImage(400,430,100,20,this.backButton);
		this.engine.drawImage(20,20,60,90,this.linkButton);
		this.engine.drawText(435,445,"Back");
		this.engine.drawText(10,200,"Pick Level");
	};
	this.pause = function(){
		this.doRender = false;
		this.doLogic = false;
	};
	this.unpause = function(){
		this.doRender = true;
		this.doLogic = true;
	};
}

function JSgGameState_CharSelect(engine, gameData){
	this.prev = null;
	this.next = null;
	this.engine = engine;

	this.gameData = gameData;

	this.bgimg = new Image();
	this.bgimg.src = 'data/test.jpg';
	this.backButton = new Image();
	this.backButton.src = 'data/button.jpg';
	this.linkButton = new Image();
	this.linkButton.src = 'data/link.jpg';

	this.init = function(){
		this.doRender = true;
		this.doLogic = true;
	};
	this.deinit = function(){
		this.doRender = false;
		this.doLogic = false;
	};
	this.logic = function(key){
		if(this.engine.mouseClicked == true){
			if(this.engine.mouseIn(400,430,100,20)){
				this.engine.popState();
			}
			if(this.engine.mouseIn(20,20,60,90)){
				this.engine.pushState(new JSgGameState_LevelSelect(this.engine,this.gameData));
			}
			this.engine.mouseClicked = false;
		}
	};
	this.render = function(){
		this.engine.drawImage(0,0,engine.WIDTH,engine.HEIGHT,this.bgimg);
		this.engine.drawImage(400,430,100,20,this.backButton);
		this.engine.drawImage(20,20,60,90,this.linkButton);
		this.engine.drawText(435,445,"Back");
		this.engine.drawText(10,200,"Pick Char");
	};
	this.pause = function(){
		this.doRender = false;
		this.doLogic = false;
	};
	this.unpause = function(){
		this.doRender = true;
		this.doLogic = true;
	};
}

function JSgGameState_MainMenu(engine){
	this.prev = null;
	this.next = null;
	this.engine = engine;

	this.gameData = new GameData(this.engine);

	this.bgimg = new Image();
	this.bgimg.src = 'data/test.jpg';
	this.playButton = new Image();
	this.playButton.src = 'data/button.jpg';

	this.init = function(){
		this.doRender = true;
		this.doLogic = true;
	};
	this.deinit = function(){
		this.doRender = false;
		this.doLogic = false;
	};
	this.logic = function(key){
		if(this.engine.mouseClicked == true){
			if(this.engine.mouseIn(400,430,100,20)){
				this.engine.pushState(new JSgGameState_CharSelect(this.engine, this.gameData));
			}
			this.engine.mouseClicked = false;
		}
	};
	this.render = function(){
		this.engine.drawImage(0,0,engine.WIDTH,engine.HEIGHT,this.bgimg);
		this.engine.drawImage(400,430,100,20,this.playButton);
		this.engine.drawText(435,445,"Play");
		this.engine.drawText(10,200,"Main Menu");
	};
	this.pause = function(){
		this.doRender = false;
		this.doLogic = false;
	};
	this.unpause = function(){
		this.doRender = true;
		this.doLogic = true;
	};
}
