/*
JSget - The 2D Full Featured Game Engine in JavaScript/HTML5

Author: Nathan Mentley (NathanMentley@GMail.com)
Version: 0.1
Updated: 10-08-2011
Web Site: http://NathanMentley.com/JSget/
File Info: JSgEngine.js contains the JSgEngine class.
License: The Creative Commons Attribution-ShareAlike 3.0 Unported License

Full License Text:
	http://creativecommons.org/licenses/by-sa/3.0/legalcode

License Summary:
	You Are Free:
	to Share - to copy, distribute and transmit the work
	to Remix - to adapt the work
	To Profit - to make commercial use of the work

	Under the contidtions:
	Attribution — You must attribute the work in the manner specified by the author or
		licensor (but not in any way that suggests that they endorse you or your use
		of the work).
	Share Alike — If you alter, transform, or build upon this work, you may distribute the
		resulting work only under the same or similar license to this one.


	With the understanding that:
	Waiver — Any of the above conditions can be waived if you get permission from the
		copyright holder.
	Public Domain — Where the work or any of its elements is in the public domain under
		applicable law, that status is in no way affected by the license.
	Other Rights — In no way are any of the following rights affected by the license:
		Your fair dealing or fair use rights, or other applicable copyright exceptions
			and limitations;
		The author's moral rights;
		Rights other persons may have either in the work itself or in how the work is
			used, such as publicity or privacy rights.
*/

//JSget's Engine Class
function JSgEngine(screenId, gamefile, firstState){
//GLOBALS
	this.JSgEngine = function(s, g, f){
		//engine globals
		this.intervalID = null;
		//canvas globals
		this.screen = document.getElementById(s);
		this.context = this.screen.getContext('2d');
		this.WIDTH = this.screen.width;
		this.HEIGHT = this.screen.height;
		//event listener globals
		this.keyArray = new Array(256);
		this.mouseX = null;
		this.mouseY = null;
		this.mouseClicked = false;
		//file manager globals
		this.gamefile = g;
		this.base64 = new Base64();

		//get and load game data archive
		xmlhttp = new XMLHttpRequest();
		xmlhttp.open("GET", this.gamefile, false);
		xmlhttp.overrideMimeType('text\/plain; charset=x-user-defined');
		xmlhttp.send();

		this.gameData = new JSUnzip(xmlhttp.responseText);
		if(!(this.gameData.isZipFile()))
			alert("error getting game data");
		this.gameData.readEntries();

		//gamestate globals
		this.currentState = null;


		//start the game.
//		f.engine = this;
//		this.loadGameData(f);
//		this.run();
	};

// DATA MANAGER FUNCTIONS
	this.loadGameData = function(game){
		this.pushState(game);
	}
	this.getFile = function(filename, type){
	//TODO: replace firefox only code to support more clients
		alert(this.gameData.entries.length);
		for (var i = 0; i < this.gameData.entries.length; i++) {
			var entry = this.gameData.entries[i];
			if(entry.fileName == filename){
				if (entry.compressionMethod === 0) {
				// Uncompressed
					var uncompressed = entry.data; 
				} else if (entry.compressionMethod === 8) {
				// Deflated
					var uncompressed = JSInflate.inflate(entry.data);
				}
				break;
			}
		}
		return "data:" + type + ";base64," + btoa(uncompressed); //+ this.base64.encode(uncompressed);
	};
	//TODO: write storedata function (store saved games or other small xml data)
	//TODO: write getdata function (get saved games or other small xml data)


// DRAWING FUNCTIONS
	this.screenClear = function(){
		this.context.clearRect(0,0,this.WIDTH,this.HEIGHT);
	};
	this.drawCircle = function(x,y,r,incolor,outcolor){
		this.context.strokeStyle = outcolor;
		this.context.fillStyle = incolor;
		this.context.beginPath();
		this.context.arc(x,y,r,0,Math.PI*2,true);
		this.context.closePath();
		this.context.stroke();
		this.context.fill();
	};
	this.drawRect = function(x,y,w,h,incolor,outcolor){
		this.context.strokeStyle = outcolor;
		this.context.fillStyle = incolor;
		this.context.beginPath();
		this.context.rect(x,y,w,h);
		this.context.closePath();
		this.context.stroke();
		this.context.fill();
	};
	this.drawText = function(x,y,text){
		this.context.fillText(text, x, y);
	};
	this.drawImage = function(x,y,w,h,img){
		this.context.drawImage(img,x,y,w,h)
	};

//AUDIO FUNCTIONS
	this.playSound = function(snd){
		snd.play();
	};
	//TODO: EXAPAND SOUND API. MUSIC AND SHITTT
	//END TODO

//PHYSICS FUNCTIONS
	//TODO: INTEGRATE BOX2dJS engine
	//END TODO

//WEBSOCKET FUNCTIONS
	//TODO: ADD WEBSOCKET SUPPORT (IF EVEN POSSIBLE)
	//END TODO

//STATE FUNCTIONS
	this.pushState = function(state){
		if(this.currentState != null){
			this.currentState.pause();
			state.prev = this.currentState;
			this.currentState.next = state;
		}
		this.currentState = state;
		this.currentState.init();
	};
	this.popState = function(){
		this.currentState.deinit();
		this.currentState = this.currentState.prev;
		if(this.currentState != null){
			this.currentState.next = null;
			this.currentState.unpause();
		}
	};
	this.runStateLogics = function(){
		var tempState = this.currentState;
		while(tempState != null){
			if(tempState.doLogic)
				tempState.logic();
			tempState = tempState.prev;
		}
	};
	this.runStateRenders = function(){
		var tempState = this.currentState;
		while(tempState != null){
			if(tempState.doRender)
				tempState.render();
			tempState = tempState.prev;
		}
	};

//USER INPUT FUNCTIONS
	this.keyPressDown = function(evt){
		engine.keyArray[evt.keyCode] = true;
	};
	this.keyPressUp = function(evt){
		engine.keyArray[evt.keyCode] = false;
	};
	this.isKeyDown = function(key){
		return engine.keyArray[key];
	};
	this.mouseOver = function(evt){
		var x;
		var y;
		if(evt.pageX || evt.pageY){
 			x = evt.pageX;
			y = evt.pageY;
		}else{
			x = evt.clientX + document.body.scrollLeft + document.documentElement.scrollLeft;
			y = evt.clientY + document.body.scrollTop + document.documentElement.scrollTop;
		}
		x -= engine.screen.offsetLeft;
		y -= engine.screen.offsetTop;

		engine.mouseX = x;
		engine.mouseY = y;
	};
	this.mouseClick = function(evt){
		engine.mouseOver(evt);
		engine.mouseClicked = true;
	};
	this.mouseIn = function(x,y,w,h){
		if((this.mouseX > x) &&
		(this.mouseX < x + w) &&
		(this.mouseY > y) &&
		(this.mouseY < y + h))
			return true;
		else
			return false;
	};

//MAIN FUNCTIONS
	this.mainloop = function(engine){
		if(engine.currentState == null){
			engine.kill();
			return;
		}

		engine.screenClear();
		engine.runStateRenders();
		engine.runStateLogics(engine.pressedKey);
	};
	this.run = function(){
		document.onkeydown = this.keyPressDown;
		document.onkeyup = this.keyPressUp;
		this.screen.onmousemove = this.mouseOver
		this.screen.addEventListener("click", this.mouseClick, false);
		this.intervalID = setInterval("engine.mainloop(engine)",33);
	};
	this.kill = function(){
		if(this.intervalID != null)
			clearInterval(this.intervalID);
		this.screenClear();
	};

	this.JSgEngine(screenId, gamefile, firstState);
}
